#include	<stdio.h>
#include	<io.h>
#include	<string.h>
#include	<process.h>
#include	<stdlib.h>
#include	<dos.h>


/*

	This program will sort the Telix Dialing Directory by BBS Name, then
	by BBS phone number.  It's simple but it works.

	If you compile it in the SMALL model, then you are limited to less
	than 700 dialing entries.  If you want to go above that, I recommend
	the HUGE model.

	The .EXE included in the original ZIP file was compiled with the HUGE
	model just incase someone had a real big dialing dorectory.

	With Turbo C use this command to compile:

		tcc -a- -mh fonsrt

*/


struct ddf_header
{
	long	id;                 /* should be hex 2e2b291a */
	int		ddf_vers;           /* currently 1 */
	int		num_entries;        /* # of entries in directory, from 1 to 1000 */
	char	pencrypted;         /* currently 0, will be used for encryption */
	char	spare[55];
};


struct dd_entry
{
	char	name[25],       /* entry name */
			number[17],     /* phone number */
			baud,           /* baud rate, see below */
			parity,         /* parity: 0 = none, 1 = even, 2 = odd */
			data,           /* number of data bits, 7 or 8 */
			stop,           /* number of stop bits, 1 or 2 */
			script[12],     /* linked script file name */
			lastcall[6];    /* last call date, stored in ASCII, w/o slashes */
	unsigned totcalls;       /* total successful calls to this entry */
	char	terminal,       /* terminal type to use, see below */
			protocol,       /* default protocol; first letter */
			toggles,        /* bit 0: local echo - 0 = off, 1 = on */
                          /* bit 1: add LFs    - 0 = off, 1 = on */
                          /* bit 2: BS trans   - 0 = destructive, 1 = not */
                          /* bit 3: BS key     - 0 = sends BS, 1 = sends DEL */
			filler1,
			filler2,
			dprefnum,       /* dialing prefix number to use when dialing */
			password[14];   /* password for this entry */
};




static int name_sort(const struct dd_entry *s, const struct dd_entry *d);
static int number_sort(const struct dd_entry *s, const struct dd_entry *d);
static void error_exit(char *s);


static struct	dd_entry	*ALL=NULL;
static			FILE		*ifp=NULL, *ofp=NULL;


main()
{
register			x;
struct	ddf_header	DDFH;
struct	dd_entry	DDE;



	printf("FONSRT - Telix Fone File Sorter ("
		   __DATE__")\n"
		   "Slapware - from The Tech Line BBS @ 301-696-1393\n\n");

	/* check command line arguments */
	if ( _argc!=3 )
		error_exit("Syntax: FONSRT <infile> <outfile>");


	/*open the input file! */
	printf("Opening input file...\n");
	if ( (ifp=fopen(_argv[1], "rb"))==NULL )
		error_exit("Opening input file!");


	/* initialize the dialing directory header */
	if ( fread(&DDFH, sizeof(struct ddf_header), 1, ifp)==0 )
		error_exit("Reading input file Header!");

	/* make sure the id is correct for v3.00 */
	if ( DDFH.id != 0x2e2b291aL )
		error_exit("Header file contains wrong id!");

	/* make sure the version is correct for v3.00 */
	if ( DDFH.ddf_vers != 1 )
		error_exit("Header file contains wrong version!");


	/* allocate space for the entries */
	if ( (ALL=malloc(sizeof (struct dd_entry) * DDFH.num_entries))==NULL )
		error_exit("Not enough memory!");

	/* open the output file! */
	printf("Creating output file...\n");
	if ( (ofp=fopen(_argv[2], "wb"))==NULL )
		error_exit("Opening output file!");


	if ( fwrite(&DDFH, sizeof(struct ddf_header), 1, ofp)==0 )
		error_exit("Writing initial output file Header!");


	printf("Header Reports %u entries...\n", DDFH.num_entries);
	x = 0;
	while ( fread(&DDE, sizeof (struct dd_entry), 1, ifp) )
	{
		memcpy(&ALL[x++], &DDE, sizeof (struct dd_entry));
		printf("\r%u read...", x);

		/* if we have read in more than we have allocated space for...
			something is wrong because we should have hit end! */
		if ( (x > DDFH.num_entries) && !feof(ifp) )
		{
			printf("\n");
			error_exit("I have read in all the entries that are reported\n"
					   "in the file header, but there seem to be more in the\n"
					   "file than the header reports!");
		}
	}


	if ( x != DDFH.num_entries )
	{
		printf("\n");
		error_exit("I have read in all the entries that are reported\n"
				   "in the file header, but there seem to be less in the\n"
				   "file than the header reports!");
	}


	printf("\rSorting entries on BBS name...\n");
	qsort(ALL, DDFH.num_entries, sizeof (struct dd_entry), name_sort);

	printf("Sorting entries on BBS number...\n");
	qsort(ALL, DDFH.num_entries, sizeof (struct dd_entry), number_sort);

	printf("Writing sorted entries to new file...\n");
	for ( x=0; x<DDFH.num_entries; x++ )
		fwrite(&ALL[x], sizeof(struct dd_entry), 1, ofp);


	printf("Closing files.... done!\n\n");

	fcloseall();
	return(0);
}




static int name_sort(const struct dd_entry *s, const struct dd_entry *d)
{
char	s1[26], d1[26];

	strncpy(s1, s->name, 25);
	strncpy(d1, d->name, 25);
	s1[25] = d1[25] = 0;

	return(strcmp(s1, d1));
}


static int number_sort(const struct dd_entry *s, const struct dd_entry *d)
{
char	s1[18], d1[18];

	strncpy(s1, s->name, 17);
	strncpy(d1, d->name, 17);
	s1[17] = d1[17] = 0;

	return(strcmp(s1, d1));
}



static void error_exit(char *s)
{
		fcloseall();			/* close all files				*/

		if ( ALL )				/* free memory block			*/
			free(ALL);

		printf("***Error: %s\n", s);


								/* remove the output file		*/
		if ( ofp!=NULL )
		{
			if ( access(_argv[2], 0)==0 )
			{
				printf("***Removing output file: %s!\n", _argv[2]);
				unlink(_argv[2]);
			}
		}

		printf("\n");

		exit(-1);
}

