#include	<stdio.h>
#include	<string.h>
#include	<process.h>
#include	<stdlib.h>

/*

	Compile this thing with Turbo C by typing "tcc -a- qm2tlx"

	The -a- tells tcc to NOT generate word alignment. This is the default
	for tcc anyway.


	I put this program together to allow me to change from Qmodem to Telix.
	Since I had been using Qmodem for 4 years, I had a lot of entries in my
	dialing directory, and I didn't want to type them all in again (even
	though it would have been quicker than writing this program).

	Anyway, it's a simple program that will convert most of your data to
	the Telix format.  It will convert the Qmodem Protocol characters to the
	different Telix equivalent.  However, remember that any External protocols
	won't be converted.  An easy fix for this is to simply have the
	same external protocol in Telix and assign it the same letter.

	An example of a (internal) protocol conversion is Qmodem uses 'G' for
	Ymodem-G, while Telix uses 'E'.

	You will also have to make new script files, and this program leaves all
	the script file name entries in the new Telix dialing directory
	blank.

*/


/*
T3FONFIL.DOC
============

   TELIX communications program v3.0 dialing directory file format.
   Copyright (c) 1988 PTel, Post Office Box 130, West Hill, Ont. M1E 4R4

   A Telix 3.0 dialing directory file is variable sized, and can hold from
   1 to 1000 entries (1000 is an arbitrary limit which might be raised, so
   do not get too dependent on it).  A dialing directory file consists of
   one ddf_header structure followed by a dd_entry structure for each
   entry iu the dialing directory.  All data is stored in Intel format.
   This means that when integers and longs are written to disk, they are
   written in the order low byte to high byte.  Since all Intel chips
   (like the 8088 and 80x86 used in PCs) store data in this fashion, this
   is of concern only if you are trying to access a directory file from a
   computer using another microprocessor, like the Mac's 68000.

   The 2 structures follow, in C format.
   All structures are packed.  No padding should be used by the compiler
   to align data on even addresses.  For example, if using the MS C
   compiler, use the -Zp switch!
*/

struct ddf_header
{
	long	id;                 /* should be hex 2e2b291a */
	int		ddf_vers;           /* currently 1 */
	int		num_entries;        /* # of entries in directory, from 1 to 1000 */
	char	pencrypted;         /* currently 0, will be used for encryption */
	char	spare[55];
};


struct dd_entry
{
	char	name[25],       /* entry name */
			number[17],     /* phone number */
			baud,           /* baud rate, see below */
			parity,         /* parity: 0 = none, 1 = even, 2 = odd */
			data,           /* number of data bits, 7 or 8 */
			stop,           /* number of stop bits, 1 or 2 */
			script[12],     /* linked script file name */
			lastcall[6];    /* last call date, stored in ASCII, w/o slashes */
	unsigned totcalls;       /* total successful calls to this entry */
	char	terminal,       /* terminal type to use, see below */
			protocol,       /* default protocol; first letter */
			toggles,        /* bit 0: local echo - 0 = off, 1 = on */
                          /* bit 1: add LFs    - 0 = off, 1 = on */
                          /* bit 2: BS trans   - 0 = destructive, 1 = not */
                          /* bit 3: BS key     - 0 = sends BS, 1 = sends DEL */
			filler1,
			filler2,
			dprefnum,       /* dialing prefix number to use when dialing */
			password[14];   /* password for this entry */
};

/*
   ASCII string fields like the name, number, password, etc. are stored in
   C format, but include the terminating 0 only if there is room. For example,
   if an entry name is 25 characters long it will completely fill up the field,
   and no terminating zero will be added. This was done for space reasons, but
   means that if you manipulate the directories you must take care. C program-
   mers for example should use the strncpy function instead of the strcpy func-
   tion to access these fields, and must make sure to add a terminating zero.

   baud field:     0 =  300, 1 =  1200, 2 =  2400, 3 =  4800,
                   4 = 9600, 5 = 19200, 6 = 38400, 7 = 57600, 8 = 115200

   terminal field: integer number from 0 - n equivalent to that entry
                   in menu shown when Alt-T function invoked
*/




main(int argc, char *argv[])
{
FILE	*ifp, *ofp;
struct	ddf_header	DDFH;
struct	dd_entry	DDE;
int		x;
long	l;
char	buf[256], tbaud[10];


	printf("QM2LX - Qmodem-to-Telix Phone Dialing Directory Converter ("
		   __DATE__")\n"
		   "Slapware - from The Tech Line BBS @ 301-696-1393\n\n");

	/* check command line arguments */
	if ( argc!=3 )
	{
		printf("Syntax: QM2TLX <infile> <outfile>\n\n");
		exit(-1);
	}


	/*open the input file! */
	printf("Opening input file...\n");
	if ( (ifp=fopen(argv[1], "rt"))==NULL )
	{
		printf("Error opening input file: %s\n\n", argv[1]);
		exit(-1);
	}

	/* open the output file! */
	printf("Creating output file...\n");
	if ( (ofp=fopen(argv[2], "wb"))==NULL )
	{
		fclose(ifp);
		printf("Error opening output file: %s\n\n", argv[2]);
		exit(-1);
	}


	/* initialize the dialing directory header */
	DDFH.id = 0x2e2b291aL;
	DDFH.ddf_vers = 1;
	DDFH.num_entries = 0;
	DDFH.pencrypted = 0;
	for (x=0; x<55; x++)
		DDFH.spare[x] = ' ';

	/* then write it to the beginning of the new dialing directory file... */
	if ( fwrite(&DDFH, sizeof(struct ddf_header), 1, ofp)==0 )
	{
		fcloseall();
		printf("Error writing directory header\n\n");
		exit(-1);
	}

	/* read from the input file til eof */
	printf("Processing file...\n");
	while ( fgets(buf, sizeof(buf), ifp) )
	{
		/* initialize the temporary baud rate string to all nulls */
		for ( x=0; x<10; x++ )
			tbaud[x]=0;


		/* Good ole sscanf() does all the work for us here.  This is a good
		   example of how powerful sscanf() can be.... */

		if ( sscanf(buf, "%*4c%25c%*6c%17c%7c%*1c%d%*1c%1c%*1c%d%*16c%14c%*2c"
						 "%2c%*1c%2c%*1c%2c %u%*3c%1c%",
				DDE.name, DDE.number, tbaud, &DDE.data, &DDE.parity, &DDE.stop,
				DDE.password, &DDE.lastcall[0], &DDE.lastcall[2], &DDE.lastcall[4],
				&DDE.totcalls, &DDE.protocol) != 12 )

			printf("Error in record %d\n", DDFH.num_entries);
		else
		{

			/* initialize some default parameters */
			DDE.filler1 = DDE.filler2 = 0;
			DDE.dprefnum = 1;
			DDE.toggles = 0;
			strcpy(DDE.script, "           ");
			DDE.terminal = 5;

			/* convert the baud rate string into a number */
			l = atol(tbaud);
			if ( l==300 )
				x = 0;
			else if ( l==1200 )
				x = 1;
			else if ( l==2400 )
				x = 2;
			else if ( l==4800 )
				x = 3;
			else if ( l==9600 )
				x = 4;
			else if ( l==19200 )
				x = 5;
			else if ( l==38400L )
				x = 6;
			else if ( l==57600L )
				x = 7;
			else if ( l==115200L )
				x = 8;
			else
				x = 5;

			DDE.baud = x;

			/* convert the parity character into a number for Telix */
			if ( DDE.parity == 'N' )
				DDE.parity = 0;
			else if ( DDE.parity == 'E' )
				DDE.parity = 1;
			else if ( DDE.parity == 'O' )
				DDE.parity = 2;
			else
				DDE.parity = 0;


			/* convert Qmodem's protocol characters into the Telix equ. */
			if ( DDE.protocol == 'C' )
				DDE.protocol = 'X';
			else if ( DDE.protocol == 'F' )
				DDE.protocol = 'G';
			else if ( DDE.protocol == 'G' )
				DDE.protocol = 'E';
			else if ( DDE.protocol == 'O' )
				DDE.protocol = '1';


			/* increment the number of entries in the dialing directory */
			DDFH.num_entries++;

			/* and write this record to the new file */
			fwrite(&DDE, sizeof(struct dd_entry), 1, ofp);

		}

	}

	/* back to the top of the file, and write a new header with the
	   new number of entries in the file */
	rewind(ofp);
	fwrite(&DDFH, sizeof(struct ddf_header), 1, ofp);

	printf("Closing files.... done!\n\n");

	fcloseall();
	return(0);
}

